@@ -21,19 +21,177 @@
 
 #include <glib.h>
 #include <string.h>
+#include <unistd.h>
 #include <locale.h>
 
-#ifndef G_OS_WIN32
-#include <unistd.h>
-#endif
-
-#include "config.h"
 #include <pango/pangocairo.h>
-#include "test-common.h"
+
+static char *
+diff_with_file (const char  *file1,
+                char        *text,
+                gssize       len,
+                GError     **error)
+{
+  const char *command[] = { "diff", "-u", file1, NULL, NULL };
+  char *diff, *tmpfile;
+  int fd;
+
+  diff = NULL;
+
+  if (len < 0)
+    len = strlen (text);
+
+  /* write the text buffer to a temporary file */
+  fd = g_file_open_tmp (NULL, &tmpfile, error);
+  if (fd < 0)
+    return NULL;
+
+  if (write (fd, text, len) != (int) len)
+    {
+      close (fd);
+      g_set_error (error,
+                   G_FILE_ERROR, G_FILE_ERROR_FAILED,
+                   "Could not write data to temporary file '%s'", tmpfile);
+      goto done;
+    }
+  close (fd);
+  command[3] = tmpfile;
+
+  /* run diff command */
+  g_spawn_sync (NULL,
+                (char **) command,
+                NULL,
+                G_SPAWN_SEARCH_PATH,
+                NULL, NULL,
+                &diff,
+                NULL, NULL,
+                error);
+
+done:
+  unlink (tmpfile);
+  g_free (tmpfile);
+
+  return diff;
+}
 
 
 static PangoContext *context;
 
+static void
+print_attr (PangoAttribute *attr, GString *string)
+{
+  g_string_append_printf (string, "[%d %d] ", attr->start_index, attr->end_index);
+  switch (attr->klass->type)
+    {
+    case PANGO_ATTR_LANGUAGE:
+      g_string_append_printf (string,"language %s\n", pango_language_to_string (((PangoAttrLanguage *)attr)->value));
+      break;
+    case PANGO_ATTR_FAMILY:
+      g_string_append_printf (string,"family %s\n", ((PangoAttrString *)attr)->value);
+      break;
+    case PANGO_ATTR_STYLE:
+      g_string_append_printf (string,"style %d\n", ((PangoAttrInt *)attr)->value);
+      break;
+    case PANGO_ATTR_WEIGHT:
+      g_string_append_printf (string,"weight %d\n", ((PangoAttrInt *)attr)->value);
+      break;
+    case PANGO_ATTR_VARIANT:
+      g_string_append_printf (string,"variant %d\n", ((PangoAttrInt *)attr)->value);
+      break;
+    case PANGO_ATTR_STRETCH:
+      g_string_append_printf (string,"stretch %d\n", ((PangoAttrInt *)attr)->value);
+      break;
+    case PANGO_ATTR_SIZE:
+      g_string_append_printf (string,"size %d\n", ((PangoAttrSize *)attr)->size);
+      break;
+    case PANGO_ATTR_FONT_DESC:
+      g_string_append_printf (string,"font %s\n", pango_font_description_to_string (((PangoAttrFontDesc *)attr)->desc));
+      break;
+    case PANGO_ATTR_FOREGROUND:
+      g_string_append_printf (string,"foreground %s\n", pango_color_to_string (&((PangoAttrColor *)attr)->color));
+      break;
+    case PANGO_ATTR_BACKGROUND:
+      g_string_append_printf (string,"background %s\n", pango_color_to_string (&((PangoAttrColor *)attr)->color));
+      break;
+    case PANGO_ATTR_UNDERLINE:
+      g_string_append_printf (string,"underline %d\n", ((PangoAttrInt *)attr)->value);
+      break;
+    case PANGO_ATTR_STRIKETHROUGH:
+      g_string_append_printf (string,"strikethrough %d\n", ((PangoAttrInt *)attr)->value);
+      break;
+    case PANGO_ATTR_RISE:
+      g_string_append_printf (string,"rise %d\n", ((PangoAttrInt *)attr)->value);
+      break;
+    case PANGO_ATTR_SHAPE:
+      g_string_append_printf (string,"shape\n");
+      break;
+    case PANGO_ATTR_SCALE:
+      g_string_append_printf (string,"scale %f\n", ((PangoAttrFloat *)attr)->value);
+      break;
+    case PANGO_ATTR_FALLBACK:
+      g_string_append_printf (string,"fallback %d\n", ((PangoAttrInt *)attr)->value);
+      break;
+    case PANGO_ATTR_LETTER_SPACING:
+      g_string_append_printf (string,"letter-spacing %d\n", ((PangoAttrInt *)attr)->value);
+      break;
+    case PANGO_ATTR_UNDERLINE_COLOR:
+      g_string_append_printf (string,"underline-color %s\n", pango_color_to_string (&((PangoAttrColor *)attr)->color));
+      break;
+    case PANGO_ATTR_STRIKETHROUGH_COLOR:
+      g_string_append_printf (string,"strikethrough-color %s\n", pango_color_to_string (&((PangoAttrColor *)attr)->color));
+      break;
+    case PANGO_ATTR_ABSOLUTE_SIZE:
+      g_string_append_printf (string,"absolute-size %d\n", ((PangoAttrSize *)attr)->size);
+      break;
+    case PANGO_ATTR_GRAVITY:
+      g_string_append_printf (string,"gravity %d\n", ((PangoAttrInt *)attr)->value);
+      break;
+    case PANGO_ATTR_GRAVITY_HINT:
+      g_string_append_printf (string,"gravity-hint %d\n", ((PangoAttrInt *)attr)->value);
+      break;
+    default:
+      g_assert_not_reached ();
+      break;
+    }
+}
+
+static void
+dump_attrs (PangoAttrList *attrs, GString *string)
+{
+  PangoAttrIterator *iter;
+
+  iter = pango_attr_list_get_iterator (attrs);
+  do {
+    gint start, end;
+    GSList *list, *l;
+
+    pango_attr_iterator_range (iter, &start, &end);
+    g_string_append_printf (string, "range %d %d\n", start, end);
+    list = pango_attr_iterator_get_attrs (iter);
+    for (l = list; l; l = l->next)
+      {
+        PangoAttribute *attr = l->data;
+        print_attr (attr, string);
+      }
+    g_slist_free_full (list, (GDestroyNotify)pango_attribute_destroy);
+  } while (pango_attr_iterator_next (iter));
+
+  pango_attr_iterator_destroy (iter);
+}
+
+static void
+dump_extra_attrs (GSList *attrs, GString *string)
+{
+  GSList *l;
+
+  for (l = attrs; l; l = l->next)
+    {
+      PangoAttribute *attr = l->data;
+
+      g_string_append (string, "  ");
+      print_attr (attr, string);
+    }
+}
 
 static const gchar *
 enum_value_nick (GType type, gint value)
@@ -60,7 +218,7 @@ direction_name (PangoDirection dir)
 static const gchar *
 gravity_name (PangoGravity gravity)
 {
-  return enum_value_nick (PANGO_TYPE_GRAVITY, gravity);
+  return enum_value_nick (PANGO_TYPE_GRAVITY, gravity);  
 }
 
 static const gchar *
@@ -98,7 +256,6 @@ dump_lines (PangoLayout *layout, GString *string)
 
   has_more = TRUE;
   index = pango_layout_iter_get_index (iter);
-  index2 = 0;
   i = 0;
   while (has_more)
     {
@@ -109,7 +266,7 @@ dump_lines (PangoLayout *layout, GString *string)
       if (has_more)
         {
           index2 = pango_layout_iter_get_index (iter);
-          char_str = g_strndup (text + index, index2 - index);
+          char_str = g_strndup (text + index, index2 - index);          
         }
       else
         {
@@ -117,7 +274,7 @@ dump_lines (PangoLayout *layout, GString *string)
         }
 
       g_string_append_printf (string, "i=%d, index=%d, paragraph-start=%d, dir=%s '%s'\n",
-                              i, index, line->is_paragraph_start, direction_name (line->resolved_dir),
+                              i, index, line->is_paragraph_start, direction_name (line->resolved_dir), 
                               char_str);
       g_free (char_str);
 
@@ -133,46 +290,57 @@ dump_runs (PangoLayout *layout, GString *string)
   PangoLayoutRun *run;
   PangoItem *item;
   const gchar *text;
-  gint index;
+  gint index, index2;
   gboolean has_more;
   gchar *char_str;
   gint i;
-  gchar *font = 0;
+  gchar *font;
 
   text = pango_layout_get_text (layout);
   iter = pango_layout_get_iter (layout);
 
   has_more = TRUE;
+  index = pango_layout_iter_get_index (iter);
   i = 0;
   while (has_more)
     {
       run = pango_layout_iter_get_run (iter);
-      index = pango_layout_iter_get_index (iter);
       has_more = pango_layout_iter_next_run (iter);
       i++;
 
+      if (has_more)
+        {
+          index2 = pango_layout_iter_get_index (iter);
+          char_str = g_strndup (text + index, index2 - index);          
+        }
+      else
+        {
+          char_str = g_strdup (text + index);
+        }
+
       if (run)
         {
           item = ((PangoGlyphItem*)run)->item;
-          char_str = g_strndup (text + item->offset, item->length);
           font = font_name (item->analysis.font);
           g_string_append_printf (string, "i=%d, index=%d, chars=%d, level=%d, gravity=%s, flags=%d, font=%s, script=%s, language=%s, '%s'\n",
                                   i, index, item->num_chars, item->analysis.level,
                                   gravity_name (item->analysis.gravity),
                                   item->analysis.flags,
-                                  "OMITTED", /* for some reason, this fails on build.gnome.org, so leave it out */
+                                  font,
                                   script_name (item->analysis.script),
                                   pango_language_to_string (item->analysis.language),
                                   char_str);
-          print_attributes (item->analysis.extra_attrs, string);
+          dump_extra_attrs (item->analysis.extra_attrs, string);
           g_free (font);
-          g_free (char_str);
         }
       else
         {
           g_string_append_printf (string, "i=%d, index=%d, no run, line end\n",
                                   i, index);
         }
+
+      g_free (char_str);
+      index = index2;
     }
   pango_layout_iter_free (iter);
 }
@@ -236,27 +404,22 @@ test_file (const gchar *filename, GString *string)
   gint ellipsize_at = 0;
   PangoEllipsizeMode ellipsize = PANGO_ELLIPSIZE_NONE;
   PangoWrapMode wrap = PANGO_WRAP_WORD;
-  PangoFontDescription *desc;
 
-  if (context == NULL)
-    context = pango_font_map_create_context (pango_cairo_font_map_get_default ());
-
-  g_file_get_contents (filename, &contents, &length, &error);
-  g_assert_no_error (error);
+  if (!g_file_get_contents (filename, &contents, &length, &error))
+    {
+      fprintf (stderr, "%s\n", error->message);
+      g_error_free (error);
+      return;
+    }
 
   p = strchr (contents, '\n');
   g_assert (p);
   markup = p + 1;
   *p = '\0';
-  length = strlen (markup);
 
   parse_params (contents, &width, &ellipsize_at, &ellipsize, &wrap);
 
   layout = pango_layout_new (context);
-  desc = pango_font_description_from_string ("Cantarell 11");
-  pango_layout_set_font_description (layout, desc);
-  pango_font_description_free (desc);
-
   pango_layout_set_markup (layout, markup, length);
   g_free (contents);
 
@@ -266,17 +429,17 @@ test_file (const gchar *filename, GString *string)
   pango_layout_set_wrap (layout, wrap);
 
   g_string_append (string, pango_layout_get_text (layout));
-  g_string_append (string, "\n--- parameters\n\n");
+  g_string_append (string, "\n---\n\n");
   g_string_append_printf (string, "wrapped: %d\n", pango_layout_is_wrapped (layout));
   g_string_append_printf (string, "ellipsized: %d\n", pango_layout_is_ellipsized (layout));
   g_string_append_printf (string, "lines: %d\n", pango_layout_get_line_count (layout));
   if (width != 0)
     g_string_append_printf (string, "width: %d\n", pango_layout_get_width (layout));
-  g_string_append (string, "\n--- attributes\n\n");
-  print_attr_list (pango_layout_get_attributes (layout), string);
-  g_string_append (string, "\n--- lines\n\n");
+  g_string_append (string, "\n---\n\n");
+   dump_attrs (pango_layout_get_attributes (layout), string);
+  g_string_append (string, "\n---\n\n");
   dump_lines (layout, string);
-  g_string_append (string, "\n--- runs\n\n");
+  g_string_append (string, "\n---\n\n");
   dump_runs (layout, string);
 
   g_object_unref (layout);
@@ -307,22 +470,6 @@ test_layout (gconstpointer d)
   GString *dump;
   gchar *diff;
 
-  const char *old_locale = setlocale (LC_ALL, NULL);
-  setlocale (LC_ALL, "en_US.utf8");
-  if (strstr (setlocale (LC_ALL, NULL), "en_US") == NULL)
-    {
-#if 0
-      // See https://github.com/mesonbuild/meson/issues/7515
-      char *msg = g_strdup_printf ("Locale en_US.UTF-8 not available, skipping layout %s", filename);
-      g_test_skip (msg);
-      g_free (msg);
-#endif
-      return;
-    }
-
-  if (context == NULL)
-    context = pango_font_map_create_context (pango_cairo_font_map_get_default ());
-
   expected_file = get_expected_filename (filename);
 
   dump = g_string_sized_new (0);
@@ -332,24 +479,13 @@ test_layout (gconstpointer d)
   diff = diff_with_file (expected_file, dump->str, dump->len, &error);
   g_assert_no_error (error);
 
-  setlocale (LC_ALL, old_locale);
-
   if (diff && diff[0])
     {
-      char **lines = g_strsplit (diff, "\n", -1);
-      const char *line;
-      int i = 0;
-
-      g_test_message ("Contents don't match expected contents");
-
-      for (line = lines[0]; line != NULL; line = lines[++i])
-        g_test_message ("%s", line);
-
+      g_printerr ("Contents don't match expected contents:\n%s", diff);
       g_test_fail ();
-      g_strfreev (lines);
+      g_free (diff);
     }
 
-  g_free (diff);
   g_string_free (dump, TRUE);
   g_free (expected_file);
 }
@@ -361,23 +497,30 @@ main (int argc, char *argv[])
   GError *error = NULL;
   const gchar *name;
   gchar *path;
+  PangoFontDescription *desc;
+
+  g_setenv ("LC_ALL", "C", TRUE);
+  setlocale (LC_ALL, "");
+
+  g_test_init (&argc, &argv, NULL);
+
+  context = pango_font_map_create_context (pango_cairo_font_map_get_default ());
+  desc = pango_font_description_from_string ("Cantarell 11");
+  pango_context_set_font_description (context, desc);
+  pango_font_description_free (desc); 
 
   /* allow to easily generate expected output for new test cases */
-  if (argc > 1 && argv[1][0] != '-')
+  if (argc > 1)
     {
       GString *string;
 
-      setlocale (LC_ALL, "en_US.utf8");
-
       string = g_string_sized_new (0);
       test_file (argv[1], string);
-      g_test_message ("%s", string->str);
+      g_print ("%s", string->str);
 
       return 0;
     }
 
-  g_test_init (&argc, &argv, NULL);
-
   path = g_test_build_filename (G_TEST_DIST, "layouts", NULL);
   dir = g_dir_open (path, 0, &error);
   g_free (path);
